/****************************************************************************/
/*                                                                          */
/*  File       :     MOUSE.C                                                */
/*  Version    :     1.0                                                    */
/*  Autor      :     Carsten Breuer                                         */
/*  First Date :     28.05.1995                                             */
/*                                                                          */
/*==========================================================================*/
/*                            R E V I S I O N                               */
/*                                                                          */
/*    Date                Beschreibung                     Ver.    Autor    */
/*  --------  ------------------------------------------  -----  ---------  */
/*  28.05.95  Mickey ist eine Tool-Sammlung fr die        1.0   C. Breuer  */
/*            Mouse-Kontrolle. Alle wichtigen                               */
/*            Funktionen zur Steuerung und Abfrage                          */
/*            der Mouse sind implementiert.                                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*                        I N F O R M A T I O N                             */
/*

In Mickey (Mouse) <g> habe ich aus den unmengen von Funktionen, die im Laufe
der Jahre entstanden sind, diejenigen kopiert, die eine einfache Steuerung
der Mouse bieten. Fr Programmierer, die gerade mit der Arbeit anfangen, ist 
die PC-Hardware oftmals ein Buch mit sieben Siegeln. So findet sich zum Thema
Maussteuerung im Borland-Handbuch kein einziger Hinweis. Erst durch weitere
Insider-Bcher erfhrt man einiges von undokumentierten DOS-Funktionen und
wie man diese anwendet. Mickey ist nicht fr den "manchmal arroganten"-
Superprogrammierer, sondern fr den Anfnger gedacht. Sie werden sehen,
wie einfach sich viele Funktionen ber Interrupts programmieren lassen.

Ich habe diese Funktionen fr Sie kopiert, da ich selber lngere Zeit nach
qualifizierten Quellen gesucht habe. Als Einsteiger-Buch kann ich Ihnen das
PC/XT/AT-Buch aus dem Sybex-Verlag empfehlen. Hier finden Sie zwar nicht
alle Informationen, aber sehr viel ber DOS- und BIOS-Interrupts und der
Hardware allgemein bis auf Chip-Ebene.

Copyright:    Mickey ist Freeware und soll es auch bleiben. Sie drfen diese
              Funktionen gerne privat oder gewerblich nutzen. Falls Sie diese
              Tools auf ein anderes BBS betragen mchten, achten Sie bitte
              darauf, da alle Dateien (Projekt, Demo, Source) bertragen
				  werden. Bitte ndern Sie den Autor-Vermerk im Header nicht.
              Teilen Sie diese Funktionen gerne mit anderen Programmieren.
              Seien Sie dabei bitte fair und geben Sie diese Funktionen
				  unendgeltlich weiter. Mickey soll helfen!

Testing:      Mickey-Funktionen laufen seit 1 Jahr bei mir ohne Probleme.
				  Sie wurden mit verschiedenen IBM, Microsoft und kompatiblen-
				  Musen getestet. Falls Sie denoch einen Fehler feststellen
				  sollten, bin ich fr Hinweise dankbar.

Infos:        Ich freue mich ber Post, die ich von Ihnen erhalte.
              Teilen Sie mir doch Ihre Erfahrungen und Probleme im
				  Zusammenhang mit Mickey und der Programmierung mit.
              Ich freue mich auch ber eine Kopie Ihres Programms,
              das mit Mickey arbeitet.



Beste Gre und viel Spa mit Mickey


Carsten Breuer
CompuServe 100427,2003

*/

#include <dos.h>
#include <conio.h>
#include <stdio.h>
#include "mouse.h"

byte MouseKeys;
byte MouseCon ;
MDAT Mouse;


/******************************************************************************
*								 H I   C O L O R
*******************************************************************************
*
*  Aufgabe     : Schaltet die oberen Hintergrundfarbe ein oder aus
*
*  Info        : HiColor ist keine Mausfunktion. Mit ihr wird das Blink-
*                bit der Videokarte abgeschaltet. Dadurch sind alle Farben
*                (auch Wei) fr den Hintergrund mglich.
*
*  Parameter   : TRUE (1) = alle Farben; FALSE (0) = Blinkbit ein (default).
*  Returnwert  : void
*
*****************************************************************************/

void HiColor (byte Mode)

{

union REGS regs;

if (Mode > 1) { Mode = 0; }

regs.h.al = 0x03;
regs.h.ah = 0x10;
regs.h.bl = Mode;

int86(0x10, &regs, &regs);

}



/******************************************************************************
*								S E T   C U R S O R   T Y P E
*******************************************************************************
*
*  Aufgabe     : Setzt den Cursor als Block oder Unterstrich
*                Na gut, auch keine Mausfunktion. Trotzdem selten!
*  Parameter   : 0 = Unterstrich; Ungleich 0 = Block
*  Returnwert  : keiner
*
*****************************************************************************/

void SetCursorType (byte Type)

{

union REGS regs;

regs.h.ah = 0x01;
regs.h.ch = 0x0D;

if (! Type)  { regs.h.cl = 0x0E; }
else         { regs.h.cl = 0x01; }

int86(0x10, &regs, &regs);         /* Interrupt 10h, Funktion 1Ah aufrufen  */

}


/******************************************************************************
*								      G E T    K E Y
*******************************************************************************
*
*  Aufgabe     : Holt eine Taste aus dem Tastaturpufffer
*                Falls keine Taste verfgbar ist wird gewartet.
*
*  Information : Im Gegensatz zu der Funktion getch() liefert GetKey die
*                Taste im 16bit-Format zurck. Es knnen somit auch
*                Tastenkombinationen wie ALT-Q abgefragt werden.
*                Mit der Funktion AltConv knnen die krummen ALT
*                Kombinationen ins normale ASCII-Format Konvertiert werden.
*
*  Parameter   : void
*  Returnwert  : Pointer auf den Viedeobereich
*
*****************************************************************************/

word GetKey (void)
{

union REGS regs;

regs.h.ah = 0x00;

int86(0x16, &regs, &regs);         /* Interrupt 16h, Funktion 00h aufrufen  */

if (! regs.h.al)                                  /* Erweiteter Tastencode  */
	{
	regs.x.ax >>= 8;
	regs.x.ax += 0xFF;
	return regs.x.ax;
	}
else               { return (word) regs.h.al; }   /* ASCII-Tastencode       */

}


/******************************************************************************
*								    A L T    C O N V
*******************************************************************************
*
*  Aufgabe     : Wandelt die durcheinander gewrfelten Alt-Tastencodes
*                in ASCCI-Codes 0-1, A-Z um..
*                Dadurch knnen die Tasten direkt mit Hotkeys verglichen
*                werden.
*
*  Parameter   : ALT-Tastencode
*  Returnwert  : Konvertierter ASCII-Code oder unverndete Taste
*
*****************************************************************************/

word AltConv (word Key)

{

switch (Key)
	{
	case ALT_Q:       { Key = 'Q'; break; }
	case ALT_W:       { Key = 'W'; break; }
	case ALT_E:       { Key = 'E'; break; }
	case ALT_R:       { Key = 'R'; break; }
	case ALT_T:       { Key = 'T'; break; }
	case ALT_Y:       { Key = 'Y'; break; }
	case ALT_U:       { Key = 'U'; break; }
	case ALT_I:       { Key = 'I'; break; }
	case ALT_O:       { Key = 'O'; break; }
	case ALT_P:       { Key = 'P'; break; }
	case ALT_A:       { Key = 'A'; break; }
	case ALT_S:       { Key = 'S'; break; }
	case ALT_D:       { Key = 'D'; break; }
	case ALT_F:       { Key = 'F'; break; }
	case ALT_G:       { Key = 'G'; break; }
	case ALT_H:       { Key = 'H'; break; }
	case ALT_J:       { Key = 'J'; break; }
	case ALT_K:       { Key = 'K'; break; }
	case ALT_L:       { Key = 'L'; break; }
	case ALT_Z:       { Key = 'Z'; break; }
	case ALT_X:       { Key = 'Z'; break; }
	case ALT_C:       { Key = 'C'; break; }
	case ALT_V:       { Key = 'V'; break; }
	case ALT_B:       { Key = 'B'; break; }
	case ALT_N:       { Key = 'N'; break; }
	case ALT_M:       { Key = 'M'; break; }
	case ALT_1:       { Key = '1'; break; }
	case ALT_2:       { Key = '2'; break; }
	case ALT_3:       { Key = '3'; break; }
	case ALT_4:       { Key = '4'; break; }
	case ALT_5:       { Key = '5'; break; }
	case ALT_6:       { Key = '6'; break; }
	case ALT_7:       { Key = '7'; break; }
	case ALT_8:       { Key = '8'; break; }
	case ALT_9:       { Key = '9'; break; }
	case ALT_0:       { Key = '0'; break; }
	default   :       { return 0 ;        }
	}
return Key;
}



/******************************************************************************
*								D E T E C T   M O U S E
*******************************************************************************
*
*  Aufgabe     : berprft ob eine Maus vorhanden ist, wenn JA Init
*  Parameter   : void
*  Returnwert  : FALSE wenn nicht, sonst Anzahl der Tasten
*
*****************************************************************************/

byte DetectMouse (void)
{

union REGS regs;

regs.x.ax = 0x0000;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 00h aufrufen  */

if (regs.x.ax == 0xFFFF)
	{
	MouseKeys = regs.x.bx;
	MouseCon  = TRUE;
	return MouseKeys;               /* Anzahl der Tasten der Maus */
	}
MouseCon = FALSE;
return 0;                          /* Keine Maus installiert     */

}



/******************************************************************************
*								D I S P L A Y    M O U S E
*******************************************************************************
*
*  Aufgabe     : Zeigt den Mouse-Cursor auf dem Bildschirm an.
*                Bei Videoausgaben mu dem Mauscursor machmal vor der
*                Ausgabe versteckt werden, da es sonst Flecken auf dem
*                Bildschirm gibt, sobald die Maus bewegt wird.
*                Der Maus-Treiber kopiert das alte Zeichen beim bewegen
*                der Maus zurck. Nicht die neue Ausgabe.
*  Parameter   : void
*  Returnwert  : void
*
*****************************************************************************/

void DisplayMouse (void)
{

union REGS regs;

if (! MouseCon) { return; }

regs.x.ax = 0x0001;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 01h aufrufen  */

}


/******************************************************************************
*								   H I D E    M O U S E
*******************************************************************************
*
*  Aufgabe     : Versteckt den Maus-Cursor
*  Parameter   : void
*  Returnwert  : void
*
*****************************************************************************/

void HideMouse (void)
{

union REGS regs;

if (! MouseCon) { return; }

regs.x.ax = 0x0002;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 02h aufrufen  */

}

/******************************************************************************
*								S E T   M O U S E    P O S
*******************************************************************************
*
*  Aufgabe     : Setzt den Maus-Cursor auf die angegebenen Koordinaten
*  Parameter   : Spalte, Zeile (Bios-Format 0,0)
*  Returnwert  : void
*
*****************************************************************************/

void SetMousePos (word sl, word zl)
{

union REGS regs;

if (! MouseCon) { return; }

regs.x.ax = 0x0004;
regs.x.cx = sl;
regs.x.dx = zl;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 04h aufrufen  */

}

/******************************************************************************
*								G E T   M O U S E   K E Y
*******************************************************************************
*
*  Aufgabe     : Ermittelt den Status der Mouse-Tasten und die aktuelle
*                Position des MouseCursors
*  Parameter   : void
*  Returnwert  : MouseTaste: 0 = None, 1 = Left, 2 = Right, 3 = Both
*
*****************************************************************************/

byte GetMouseKey (void)
{
union REGS regs;

Mouse.Spalte  = 0;
Mouse.Zeile   = 0;
Mouse.Key     = 0;

if (! MouseCon) { return 0; }

regs.x.ax = 0x0003;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 03h aufrufen  */

Mouse.Spalte  = regs.x.cx / 8;
Mouse.Zeile   = regs.x.dx / 8;
Mouse.Key     = regs.x.bx;

return Mouse.Key;
}


/******************************************************************************
*								G E T   M O U S E    I N F O
*******************************************************************************
*
*  Aufgabe     : Ermittelt den Typ der Maus und die Version des Maus-Treibers.
*
*  Information : Diese Funktion ist nicht bei allen Maustreibern verfgbar!
*
*  Parameter   : void
*  Returnwert  : String mit den Mausdaten
*
*****************************************************************************/

char *GetMouseInfo (void)
{

union REGS regs;
static char  str[25];

char  Ver[5][8] =   {
							{"Bus"},
							{"Seriell"},
							{"Inport"},
							{"PS/2"},
							{"HP"},
						  };

if (! MouseCon) { return '\0'; }

regs.x.ax = 0x0024;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 24h aufrufen  */

sprintf (str, "%s Version: %u.%u", Ver[regs.h.ch - 1], regs.h.bh, regs.h.bl);

return str;
}


/******************************************************************************
*								S E T   M O U S E    A R E A
*******************************************************************************
*
*  Aufgabe     : Setzt den Maus-Bewegungsspielraum auf die angegebenen
*                Koordinaten.
*
*  Information : Die Parameter werden unten mit 8 multipliziert.
*                Dies hngt mit dem Standard 8x8-Zeichensatz des PC's
*                zusammen. Der Mauscursor hat eine 8mal hhere Auflsung,
*                als auf dem Bildschirm sichtbar. Ein Wechsel von einem
*                Zeichen entspricht 8 - 15 Mausticks.
*
*  Parameter   : void
*  Returnwert  : String mit den Mausdaten
*
*****************************************************************************/

void SetMouseArea (word sl, word zl, word sr, word zr)

{

union REGS regs;

if (! MouseCon)     { return; }

if (sl >= sr)       { return; }
if (zl >= zr)       { return; }
if (sr >= 80)       { return; }
if (zl >= 25)       { return; }

regs.x.ax = 0x0007;
regs.x.cx = sl * 8;
regs.x.dx = sr * 8;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 07h aufrufen  */

regs.x.ax = 0x0008;
regs.x.cx = zl * 8;
regs.x.dx = zr * 8;

int86(0x33, &regs, &regs);         /* Interrupt 33h, Funktion 07h aufrufen  */

}




